; comparator_osc_display_jan11.a51   comparators watch charging cap, make oscillation;  
;  assumes photodiode (or, simpler, variable R) between V+ and cap, monitored by compar + input

; Jan. 11 add display of hi byte of 16-bit accumulation of comparator edges
;  re-assign MOSFET gate to P0.3, and let Wizard handle configuration

; rev path 3/16
	$NOSYMBOLS		; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file

	STACKBOT EQU 80h	; put stack at start of scratch indirectly-addressable block (80h and up)

  COMP0_INT EQU 063h				; address of comparator-zero interrupt vector

; PORT USE:
; comparator inputs: - is P0.6, + is P0.7
; discharge MOSFET: P0.3
; LCD PORT1
; keypad PORT2
; OVERFLOW LED P0.0	; lights when exposure is done (count matches keypad)
	LED EQU P0.0

; REGISTER USE:
;	R7 hi byte of oscillation-count
;	R6 lo byte of osc-count


	ORG 0h
	LJMP STARTUP

	ORG 100h
				
STARTUP: MOV SP, #STACKBOT-1

	ACALL USUAL_SETUP

	ACALL MISC_INITS
	ACALL Init_Device     ; this includes interrupt enable, so comes last

OSC_LOOP:	MOV A, CPT0CN			; get this where we can test rising-edge flag
  JNB ACC.6, OSC_LOOP		; branch on comparator output (not flag)
	SETB P0.3		; discharge cap (turn on MOSFET)
	ACALL delay		; ...allow enough time for full discharge of cap
	CLR P0.3		; ...turn off MOSFET
	MOV P1, R7		; display hi byte of cycle counter

CHECK_END: MOV A, R7	; get current count for comparison
	CJNE A, P2, OSC_LOOP	; if not yet done, keep counting
DONE: CLR LED			; light LED indicating that tanning exposure is complete
	SJMP $			; hang here till interrupted

;------------------------------
; ISR for comparator interrupt
	ORG COMP0_INT

	ANL CPT0CN, #0DFh	; clear comparator interrupt flag
	LJMP COMPARATOR_INT_RESPONSE

	ORG 150h
COMPARATOR_INT_RESPONSE:	PUSH ACC   ; save scratch register
SIXTEEN_BIT_INC:	MOV A, #1	    ; set up an increment that will affect CY flag
	ADD A, R6			; increment low byte of count
	MOV R6, A
	MOV A, R7			; get hi byte
	ADDC A, #0			; increment hi byte if there was a carry from low byte
	MOV R7, A
	POP ACC
	RETI
;--------------------------------


;------- INITIALIZATIONS
MISC_INITS: 	CLR P0.3			; MOSFET off
	SETB LED				; overflow (tan done) LED off
	SETB P0.6				; write One's to pins to be used as inputs
	SETB P0.7
	MOV R7, #0			; clear osc count
	MOV R6, #0
	RET

USUAL_SETUP: ANL   PCA0MD, #NOT(040h)        ; Disable the WDT.
                                             ; Clear Watchdog Enable bit
	; Configure the Oscillator
        ORL   OSCICN, #04h         ; sysclk = 24.5 Mhz / 8
	RET

		
DELAY: PUSH ACC
			MOV A, #80h	; works pretty well: 10h caused step up on release of reset
			DJNZ ACC,$
			POP ACC
			RET

; Called from the Init_Device label
Comparator_Init:
    mov  CPT0CN,    #085h    ; enable comparator 0; output high if + > - ; hysteresis +- 5 mV
;    clr  A                     ; Wait 10us for initialization
;    djnz ACC, $             ; THIS IS A BIT PERFECTIONIST, FOR OUR PURPOSES!
    anl  CPT0CN,    #0CFh    ; clear two Comp0 flags (both rising and falling edge)--bars interrupt
    mov  CPT0MX,    #033h    ; establish input pins for comparators (P0.6, P0.)
    mov  CPT0MD,    #023h    ; lowest power, lowest speed setting for comparator; int on rising edge 
    ret


Port_IO_Init:
; we can omit these skips
    ; P0.0  -  Skipped,     Open-Drain, Digital
    ; P0.1  -  Skipped,     Open-Drain, Digital
    ; P0.2  -  Skipped,     Open-Drain, Digital
    ; P0.3  -  Skipped,     Push-Pull,  Digital
    ; P0.4  -  Skipped,     Open-Drain, Digital
    ; P0.5  -  Skipped,     Open-Drain, Digital
    ; P0.6  -  CP0 (Cmp0),  Open-Drain, Analog
    ; P0.7  -  CP0A (Cmp0), Open-Drain, Analog

    mov  P0MDIN,    #03Fh    ; make the two comparator inputs analog (P0.6, P0.7)
    mov  P0MDOUT,   #008h    ; make P0.3 push-pull, to drive MOSFET gate
    mov  P0SKIP,    #03Fh    ; skip portpins so as to assign comparator inputs to P0.6, P0.7
    mov  XBR1,      #040h    ; enable crossbar
    ret

Interrupts_Init:
    mov  EIE1,      #020h    ; enable interrupt on rising edge of Comp0 (flag)
    mov  IE,        #080h    ; global interrupt enable
    ret

; Initialization function for device,
; Call Init_Device from your main program
Init_Device:
    lcall Comparator_Init
    lcall Port_IO_Init
    lcall Interrupts_Init
    ret

END
	
